@@ -1,7 +1,7 @@
 /*
  *  Authors: Rodney Dawes <dobey@ximian.com>
  *
- *  Copyright 2004 Novell, Inc. (www.novell.com)
+ *  Copyright 2004-2007 Novell, Inc. (www.novell.com)
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of version 2 of the GNU General Public License
@@ -20,6 +20,7 @@
 
 #include <config.h>
 #include <gdk/gdkcolor.h>
+#include <libbonobo.h>
 
 #include "evolution-webcal-notify.h"
 
@@ -47,11 +48,46 @@ enum {
   E_ICALSHARE_TIMEOUT_WEEKS
 };
 
+static GtkWidget * e_webcal_label_new (const gchar * str) {
+  GtkWidget * label;
+
+  label = gtk_label_new (str);
+  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+
+  return label;
+}
+
+static void e_webcal_label_size_allocate_cb (GtkWidget *widget,
+					     GtkAllocation *allocation,
+					     gpointer user_data) {
+  gint width, height;
+  GtkWidget *label;
+
+  label = user_data;
+
+  gtk_widget_get_size_request (label, &width, &height);
+
+  if (allocation->width > 1 &&
+      allocation->width != (width + label->allocation.x)) {
+    gtk_widget_set_size_request (label,
+				 allocation->width - label->allocation.x,
+				 -1);
+    gtk_container_check_resize (GTK_CONTAINER (widget));
+  }
+}
+
+static void e_webcal_label_handle_resize (GtkWidget * label,
+					  GtkWidget * container) {
+  g_signal_connect (G_OBJECT (container), "size-allocate",
+		    G_CALLBACK (e_webcal_label_size_allocate_cb), label);
+}
+
 static void e_webcal_icon_theme_changed (GnomeIconTheme * theme,
 					 EIcalShareDialog * dialog) {
   gchar * icofile;
 
-  icofile = gnome_icon_theme_lookup_icon (dialog->theme, "stock_web-calendar",
+  icofile = gnome_icon_theme_lookup_icon (dialog->theme, "x-office-calendar",
 					  48, NULL, NULL);
 
   if (icofile != NULL) {
@@ -163,6 +199,7 @@ void e_webcal_display_error (const gchar * title, const gchar * message,
 
   label = gtk_label_new (message);
   gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+  gtk_label_set_max_width_chars (GTK_LABEL (label), 80);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
@@ -212,7 +249,7 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   GtkWidget * rlabel, * clabel;
   GtkWidget * cbutton;
   GtkWidget * label, * button;
-  SoupUri * tmpuri;
+  SoupURI * tmpuri;
   gchar * tmpname;
   gchar * icofile, * mrkname, * ref_str;
   gint ref_timeout, ref_multi;
@@ -225,6 +262,26 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   gchar * scolor;
   guint32 icolor = 0;
 
+  tmpuri = soup_uri_new (caluri);
+  tmpname = g_path_get_basename (tmpuri->path);
+
+  /* If we are already subscribed, pop up an error dialog */
+  if (!has_events && !has_tasks) {
+    gchar * msg;
+
+    msg = g_strdup_printf (_("No events or tasks were found in the calendar "
+			     "\"%s\"."),
+			   name ? name : tmpname);
+    e_webcal_display_error (_("<b>No Tasks or Events Found</b>"), msg, NULL);
+    g_free (msg);
+
+    g_free (tmpname);
+    soup_uri_free (tmpuri);
+    
+    bonobo_main_quit ();
+    return;
+  }
+
   events_sources = e_source_list_new_for_gconf_default (CALENDAR_SOURCES_KEY);
 
   for (l = e_source_list_peek_groups (events_sources); l != NULL; l = l->next) {
@@ -255,9 +312,6 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
     e_source_list_add_group (tasks_sources, tasks_group, -1);
   }
 
-  tmpuri = soup_uri_new (caluri);
-  tmpname = g_path_get_basename (tmpuri->path);
-
   if (has_events)
     if (e_source_group_peek_source_by_name (events_group,
 					    name ? name : tmpname)) {
@@ -274,7 +328,7 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   if (!has_events && !has_tasks) {
     gchar * msg;
 
-    msg = g_strdup_printf (_("You are already subscribed to the calendar, "
+    msg = g_strdup_printf (_("You are already subscribed to the calendar "
 			     "\"%s\". Please subscribe to another calendar."),
 			   name ? name : tmpname);
     e_webcal_display_error (_("<b>Already Subscribed</b>"), msg, NULL);
@@ -307,7 +361,7 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
   gtk_window_set_default_size (GTK_WINDOW (dialog->dialog), 376, 144);
 
   icofile = gnome_icon_theme_lookup_icon (dialog->theme,
-					  "stock_web-calendar",
+					  "x-office-calendar",
 					  48, NULL, NULL);
   if (icofile != NULL) {
     gtk_window_set_default_icon_from_file (icofile, NULL);
@@ -373,22 +427,23 @@ void e_webcal_query_user (const gchar * name, const gchar * desc,
 
   /* Box for Name and Description */
   vbox = gtk_vbox_new (FALSE, 12);
-  gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);
   gtk_widget_show (vbox);
 
   /* Name */
   mrkname = g_strdup_printf ("<b>%s</b>", name ? name : tmpname);
-  label = gtk_label_new (mrkname);
-  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+  label = e_webcal_label_new (mrkname);
+  e_webcal_label_handle_resize (label, vbox);
   g_free (mrkname);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
   /* Description */
-  label = gtk_label_new (desc ? desc : _("No Description"));
+  label = e_webcal_label_new (desc ? desc : _("No Description"));
+  e_webcal_label_handle_resize (label, vbox);
+  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
   gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-  gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
   gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
   gtk_widget_show (label);
 
